@@ -91,6 +91,10 @@ class Autofix_Instruction:
         # Classification if this token is a unary or binary
         # operator. Only set for OPERATOR tokens.
 
+        self.spurious = False
+        # Classification for spurious tokens. Specifically this can be
+        # set on commas so that mh_style can remove them.
+
         self.statement_terminator = False
         # Classification if this comma/semicolon token actually ends a
         # statement. I.e. not true for the punctuation inside matrices
@@ -1396,8 +1400,8 @@ class Identifier(Name):
         assert isinstance(cfg, Config)
 
         if self.t_ident.value in HIGH_IMPACT_BUILTIN_FUNCTIONS:
-            mh.style_issue(self.t_ident.location,
-                           "redefining this builtin is very naughty")
+            mh.check(self.t_ident.location,
+                     "redefining this builtin is very naughty")
 
 
 class Selection(Name):
@@ -2031,20 +2035,33 @@ class Import_Statement(Simple_Statement):
                 for t in self.l_chain]
 
 
-class Metric_Justification_Pragma(Pragma):
-    def __init__(self, t_pragma, t_kind, t_tool, t_metric, n_reason):
+class Justification_Pragma(Pragma):
+    def __init__(self, t_pragma, t_kind, t_tool):
         super().__init__(t_pragma, t_kind)
         assert isinstance(t_tool, MATLAB_Token)
-        assert t_tool.kind == "IDENTIFIER" and t_tool.value == "metric"
-        assert isinstance(t_metric, MATLAB_Token)
-        assert t_metric.kind == "STRING"
-        assert isinstance(n_reason, (String_Literal, Binary_Operation))
+        assert t_tool.kind == "IDENTIFIER"
 
         self.t_tool = t_tool
         self.t_tool.set_ast(self)
         # The tool. Will always be 'metric' in this case. This will
         # move out to general justification pragmas once we get there.
 
+        self.applies = False
+        # Is set to true by mh_metric if this pragma successfully
+        # justifies a metrics violation. That way we can do a tree
+        # walk at the end and complain about all pragmas that don't
+        # actually do something.
+
+
+class Metric_Justification_Pragma(Justification_Pragma):
+    def __init__(self, t_pragma, t_kind, t_tool, t_metric, n_reason):
+        super().__init__(t_pragma, t_kind, t_tool)
+        assert isinstance(t_tool, MATLAB_Token)
+        assert t_tool.kind == "IDENTIFIER" and t_tool.value == "metric"
+        assert isinstance(t_metric, MATLAB_Token)
+        assert t_metric.kind == "STRING"
+        assert isinstance(n_reason, (String_Literal, Binary_Operation))
+
         self.t_metric = t_metric
         self.t_metric.set_ast(self)
         # A string to identify the metric
@@ -2054,12 +2071,6 @@ class Metric_Justification_Pragma(Pragma):
         # The reason why this deviation is OK. Currently just a string
         # literal, but can be a string expression in the future.
 
-        self.applies = False
-        # Is set to true by mh_metric if this pragma successfully
-        # justifies a metrics violation. That way we can do a tree
-        # walk at the end and complain about all pragmas that don't
-        # actually do something.
-
     def metric(self):
         return self.t_metric.value
 
