@@ -1138,9 +1138,9 @@ class MATLAB_Parser:
         self.amatch("IDENTIFIER")
         t_pragma_kind = self.ct
         if t_pragma_kind.value not in ("Justify", ):
-            self.mh.warning(t_pragma_kind.location,
-                            "unknown miss_hit pragma '%s'" %
-                            t_pragma_kind.value)
+            self.mh.error(t_pragma_kind.location,
+                          "unknown miss_hit pragma '%s'" %
+                          t_pragma_kind.value)
 
         self.amatch("BRA")
         punctuation.append(self.ct)
@@ -1148,9 +1148,9 @@ class MATLAB_Parser:
         self.amatch("IDENTIFIER")
         t_tool = self.ct
 
-        if t_tool.value not in ("metric" ,):
-            self.mh.warning(t_tool.location,
-                            "unknown miss_hit tool '%s'" % t_tool.value)
+        if t_tool.value != "metric":
+            self.mh.error(t_tool.location,
+                          "unknown miss_hit tool '%s'" % t_tool.value)
 
         self.amatch("COMMA")
         punctuation.append(self.ct)
@@ -1175,6 +1175,7 @@ class MATLAB_Parser:
 
         rv = Metric_Justification_Pragma(t_pragma, t_pragma_kind,
                                          t_tool, t_param, n_reason)
+
         for token in punctuation:
             token.set_ast(rv)
 
@@ -1521,9 +1522,10 @@ class MATLAB_Parser:
             rv = Binary_Operation(8, t_op, rv, rhs)
 
             if chain_length > 2:
-                self.mh.warning(t_op.location,
-                                "chained relation does not work the"
-                                " way you think it does")
+                self.mh.check(t_op.location,
+                              "chained relation does not work the"
+                              " way you think it does",
+                              "high")
 
         return rv
 
@@ -1595,14 +1597,7 @@ class MATLAB_Parser:
                 if self.peek("COMMA"):
                     self.match("COMMA")
                     self.ct.set_ast(rv)
-
-            if (self.peek("SEMICOLON") or
-                self.peek("NEWLINE") or
-                self.peek("C_KET") or
-                self.peek("M_KET")):
-                # Bad style, but you can have a trailing comma in your
-                # matrix, e.g. [1,2,] which is the same as [1, 2]
-                break
+                    self.ct.fix.spurious = True
 
             rv.add_item(self.parse_nested_expression())
 
@@ -1616,6 +1611,16 @@ class MATLAB_Parser:
                 self.match("COMMA")
                 self.ct.set_ast(rv)
 
+                if (self.peek("SEMICOLON") or
+                    self.peek("NEWLINE") or
+                    self.peek("C_KET") or
+                    self.peek("M_KET")):
+                    # Bad style, but you can have a trailing comma in
+                    # your matrix, e.g. [1,2,] which is the same as
+                    # [1, 2]. We specifically detect it here, so we
+                    # can possibly issue a message.
+                    self.ct.fix.spurious = True
+
         return rv
 
     def parse_matrix(self):
